# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .database import PICUDatabase


class UnicodeDataVersionManager(object):
    """
    Manage multiple version of UnicodeDatabase objects.

    Note: A UnicodeDataVersionManager instance can only manage
    UnicodeDatabase of the same class.
    """

    def __init__(self, database_class=PICUDatabase):
        self._versions = {}
        self._database_class = database_class

    def register(self, expected_version, *args, **kwargs):
        """
        Create and register a new UnicodeDatabase in the manager.

        :param expected_version: The expected Unicode version.
        :returns: The created UnicodeDatabase object.
        :raises ValueError: If expected_version and actual Unicode version
                            of the created database mismatch.
        """
        db = self._database_class(*args, **kwargs)
        loaded_ver = db.get_unicode_version()
        if expected_version and loaded_ver != expected_version:
            raise ValueError("unicode version mismatch ('{0}' != '{1}')".format(loaded_ver, expected_version))
        self._versions[loaded_ver] = db
        return db

    def get_db_by_version(self, unicode_version):
        """
        Retrieve a UnicodeDatabase instance by its version.

        :param unicode_version: The Unicode version of the database.
        :returns: The instance of the UnicodeDatabase
                  whose version is unicode_version.
        :raises ValueError: If no corresponding database has been registered.
        """
        return self._versions[unicode_version]



#!/usr/bin/env python

import sys
import os
from picu.loader import icu_load
from picu.constants import U_UNICODE_CHAR_NAME, U_SHORT_PROPERTY_NAME

__version__ = '0.1'
__author__ = 'Wil Tan <wil@cloudregistry.net>'


def main():
    import argparse
    import logging

    parser = argparse.ArgumentParser(description='Test script working with multiple versions of ICU')
    parser.add_argument('-v', '--verbose', action='store_true', help='be verbose')
    parser.add_argument('--libs', action='append', default=[])
    parser.add_argument('--version', action='version', version='%(prog)s {}'.format(__version__))
    parser.add_argument('codepoint', metavar='CP')
    parser.add_argument('property', metavar='PROP')

    args = parser.parse_args()

    logging.basicConfig(stream=sys.stderr, level=logging.DEBUG if args.verbose else logging.INFO)

    script_name = sys.argv[0]
    if script_name.endswith('.py'):
        script_name = script_name[:-3]
    logger = logging.getLogger(script_name)

    if not args.libs:
        libpath = os.getenv('PICU_ICU_LIB_PATH', '/usr/local/Cellar/icu4c/52.1/lib/libicuuc.52.1.dylib')
        i18n_libpath = os.getenv('PICU_ICU_I18N_LIB_PATH', '/usr/local/Cellar/icu4c/52.1/lib/libicui18n.52.1.dylib')
        libver = os.getenv('PICU_ICU_LIB_VER', '52')
        args.libs.append('{}#{}#{}'.format(libpath, i18n_libpath, libver))

    icu_libs = []
    for libs in args.libs:
        libpath, i18n_libpath, libver = libs.split('#')
        icu = icu_load(libpath, i18n_libpath, libver)
        icu_libs.append(icu)

        logger.info("Library '%s' loaded. Unicode version: %s", libpath, icu.getUnicodeVersion())
        # call the main workhorse function

    if not icu_libs:
        logging.error("No --libs given")
        sys.exit(1)

    cp = int(args.codepoint, 16)
    prop = args.property
    for icu in icu_libs:
        logger.info("Unicode version: %s", icu.getUnicodeVersion())
        print("U+%04X (%s)" % (cp, icu.charName(cp, U_UNICODE_CHAR_NAME)))
        if prop.lower() == 'script_extensions':
            print(icu.get_script_extensions(cp))
        elif prop.lower() == 'script':
            print(icu.get_script(cp))
        elif prop.lower() == 'script_alpha4':
            print(icu.get_script(cp, U_SHORT_PROPERTY_NAME))
        elif prop.lower() == 'age':
            print(icu.charAge(cp))
        else:
            print(icu.get_prop_value(cp, prop))


if __name__ == '__main__':
    main()



#!/usr/bin/env python

import sys
import os
from munidata import UnicodeDataVersionManager

__version__ = '0.1'
__author__ = 'Wil Tan <wil@cloudregistry.net>'


def main():
    import argparse

    parser = argparse.ArgumentParser(description='Test script working with multiple versions of ICU')
    parser.add_argument('-v', '--verbose', action='store_true', help='be verbose')
    parser.add_argument('--libs', action='append', default=[])
    parser.add_argument('--version', action='version', version='%(prog)s {}'.format(__version__))
    parser.add_argument('codepoint', metavar='CP')
    parser.add_argument('property', metavar='PROP')

    args = parser.parse_args()

    manager = UnicodeDataVersionManager()
    if not args.libs:
        libpath = os.getenv('PICU_ICU_LIB_PATH', '/usr/local/Cellar/icu4c/52.1/lib/libicuuc.52.1.dylib')
        i18n_libpath = os.getenv('PICU_ICU_I18N_LIB_PATH', '/usr/local/Cellar/icu4c/52.1/lib/libicui18n.52.1.dylib')
        libver = os.getenv('PICU_ICU_LIB_VER', '52')
        args.libs.append('{}#{}#{}'.format(libpath, i18n_libpath, libver))

    databases = []
    for libs in args.libs:
        libpath, i18n_libpath, libver = libs.split('#')
        db = manager.register(None, libpath, i18n_libpath, libver)
        databases.append(db)

    if not databases:
        sys.exit(1)

    cp = int(args.codepoint, 16)
    prop = args.property
    for db in databases:
        print("Unicode version: %s" % db.get_unicode_version())
        print("U+%04X (%s)" % (cp, db.get_char_name(cp)))
        if prop.lower() == 'script_extensions':
            print(db.get_script_extensions(cp))
        elif prop.lower() == 'script_alpha4':
            print(db.get_script(cp, alpha4=True))
        elif prop.lower() == 'age':
            print(db.get_char_age(cp))
        else:
            print(db.get_prop_value(cp, prop))

if __name__ == '__main__':
    main()



#!/bin/env python2
# -*- coding: utf-8 -*-
"""
parse_idna_tables.py - Parse IDNA2008 tables to list codepoint property.

Parse the IDNA 2008 tables from IANA website and create a dictionnary with all
codepoints and their IDNA property.

Copied from https://github.com/kjd/idna/blob/master/tools/build-idnadata.py
"""
from lxml import etree
import sys
import argparse
import logging

IDNATABLES_URL = "http://www.iana.org/assignments/idna-tables-{version}/idna-tables-{version}.xml"
IDNATABLES_NS = "http://www.iana.org/assignments"

CODEPOINT_TAG = "{%s}codepoint" % IDNATABLES_NS
PROPERTY_TAG = "{%s}property" % IDNATABLES_NS

logger = logging.getLogger(__name__)


def print_optimised_list(prop):
    codepoint_list = sorted(prop)
    single_codepoints = []
    set_elements = []
    last_write = -1
    for i in xrange(0, len(codepoint_list)):
        if i + 1 < len(codepoint_list):
            if codepoint_list[i] == codepoint_list[i+1]-1:
                continue
        codepoint_range = codepoint_list[last_write+1:i+1]
        if len(codepoint_range) == 1:
            single_codepoints.append("{0}".format(hex(codepoint_range[0])))
        else:
            set_elements.append("list(range({0},{1}))".format(codepoint_range[0],
                                                              codepoint_range[-1] + 1))
        last_write = i

    print("        frozenset(")
    if len(single_codepoints) > 0:
        print("        [\n         " + ",\n         ".join(single_codepoints))
        print("        ] +")
    print("        " + " +\n        ".join(set_elements))
    print("    ),")


def parse_idna_tables(version):
    """
    Make the list of codepoints from IDNA tables.

    Parse IDNA table registry, and create a mapping of all codepoints to their
    structure.

    Input:
        * version: The unicode version to use.
    """
    idna_url = IDNATABLES_URL.format(version=version)
    short_ver = version.translate(None, '.')
    logger.debug("Fetching and parsing '%s'", idna_url)
    registry = etree.parse(idna_url)

    # To keep '{}' when string-formatting
    namespace = "{{{0}}}".format(IDNATABLES_NS)
    registry_id = "idna-tables-properties"
    if list(map(int, version.split('.'))) <= [6, 0, 0]:
        registry_id = "idna-tables-{}-properties".format(version)
    record_xpath = '{0}registry[@id="{1}"]/{0}record'.format(namespace,
                                                             registry_id)

    print('# -*- coding: utf-8 -*-')
    print('# File generated by parse_idna_tables tool')
    print('# Unicode version: %s' % version)
    print('# Generated from %s' % idna_url)
    print('')
    print('idna_tables_%s = {' % short_ver)

    properties = {}
    for record in registry.findall(record_xpath):
        codepoint = record.find(CODEPOINT_TAG).text
        prop = record.find(PROPERTY_TAG).text

        if prop == 'UNASSIGNED':
            continue

        if prop not in properties:
            properties[prop] = set()

        if codepoint.find('-') > 0:
            # Codepoint is a range
            (first_cp, last_cp) = [int(c, 16) for c in codepoint.split('-')]
            for cp in xrange(first_cp, last_cp + 1):
                properties[prop].add(cp)
        else:
            # Single codepoint
            properties[prop].add(int(codepoint, 16))

    for prop in properties:
        print("    '{0}':".format(prop))
        print_optimised_list(properties[prop])

    print('}')


def main():
    parser = argparse.ArgumentParser(description='Parse and make '
                                                 'IDNA properties '
                                                 'from IDNA2008 registry')
    parser.add_argument('-v', '--verbose', action='store_true', help='be verbose')
    parser.add_argument('unicode', metavar='Unicode Version')

    args = parser.parse_args()

    logging.basicConfig(stream=sys.stderr, level=logging.DEBUG if args.verbose else logging.INFO)

    parse_idna_tables(args.unicode)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from picu.loader import icu_load
from picu.constants import U_LONG_PROPERTY_NAME, U_SHORT_PROPERTY_NAME
from .idna import idnatables


class UnicodeDatabase(object):
    """
    Base interface and implementation of a Unicode Database.
    """
    def get_unicode_version(self):
        """
        Get the Unicode Version handled by this instance.
        """
        raise NotImplementedError

    def get_char_name(self, cp):
        """
        Retrieve the character's name.

        :param cp: Codepoint.
        :returns: The character's name.
        """
        raise NotImplementedError

    def get_char_age(self, cp):
        """
        Retrieve the character's age.

        :param cp: Codepoint.
        :returns: The character's age.
        """
        raise NotImplementedError

    def get_script(self, cp, alpha4=False):
        raise NotImplementedError

    def get_script_extensions(self, cp, alpha4=False):
        raise NotImplementedError

    def get_prop_value(self, cp, prop_name, prop_type=U_LONG_PROPERTY_NAME):
        raise NotImplementedError

    def is_combining_mark(self, cp):
        """
        Check if character is a combining mark

        :param cp: Codepoint.
        :return:  Whether the character is a combining mark.
        """
        raise NotImplementedError

    def is_digit(self, cp):
        """
        Check if character is a digit.

        :param cp: Codepoint.
        :return:  Whether the character is a digit.
        """
        raise NotImplementedError

    def is_script_rtl(self, script):
        """
        Check is a script is Right To Left.

        :param script: Script.
        :return: Whether the script is Right To Left.
        """
        raise NotImplementedError

    def get_idna_prop(self, cp):
        """
        Retrieve the IDNA property of a character.

        :param cp: Codepoint.
        :returns: IDNA property of the character.
        :raises ValueError: If the Unicode Version is not handled
                            by the IDNA module.
        """
        return idnatables.get_idna_property(cp,
                                            self.get_unicode_version())

    def compile_regex(self, regex):
        """
        Compile a regex.

        :param regex: The regex as a string.
        :return: A regex object, implementation-dependant.
        """
        raise NotImplementedError

    def get_set(self, iterable=None, pattern=None, freeze=False):
        """
        Get a set from given parameters.

        :param iterable: Optional iterable to convert to a set.
        :param pattern: Optional pattern to convert to a set.
        :param freeze: Optional flag to indicate if the set is frozen.
        :return: A set object, implementation-dependant.
        """
        raise NotImplementedError

    def idna_encode(self, input, options=None):
        """
        Encode the given domain name according to the IDNA2008 protocol

        :param input: Unicode string of the domain name to be encoded
        :param options: Optional bitmask options from `picu.constants.UIDNA_*`
        :return: A unicode string representing the result of encoding
        """
        raise NotImplementedError

    def idna_decode(self, input, options=None):
        """
        Decode the given domain name according to the IDNA2008 protocol

        :param input: Unicode string of the domain name to be decoded
        :param options: Optional bitmask options from `picu.constants.UIDNA_*`
        :return: A unicode string representing the result of decoding
        """
        raise NotImplementedError

    def idna_encode_label(self, input, options=None):
        """
        Encode the given domain name label according to the IDNA2008 protocol

        :param input: Unicode string of the domain name label to be encoded
        :param options: Optional bitmask options from `picu.constants.UIDNA_*`
        :return: A unicode string representing the resulting A-label
        """
        raise NotImplementedError

    def idna_decode_label(self, input, options=None):
        """
        Decode the given domain name labelaccording to the IDNA2008 protocol

        :param input: Unicode string of the domain name label to be decoded
        :param options: Optional bitmask options from `picu.constants.UIDNA_*`
        :return: A unicode string representing the resulting U-label
        """
        raise NotImplementedError


class IDNADatabase(UnicodeDatabase):
    """
    Simple implementation that only supports IDNA properties.
    """

    def __init__(self, version):
        self.version = version

    def get_unicode_version(self):
        return self.version


class PICUDatabase(UnicodeDatabase):
    """
    Complete implementation using the PICU python module.
    """

    def __init__(self, icu_uc_lib, icu_i18n_lib, version_tag):
        self._icu = icu_load(icu_uc_lib, icu_i18n_lib, version_tag)

    def get_unicode_version(self):
        return self._icu.getUnicodeVersion()

    def get_char_name(self, cp):
        return self._icu.charName(cp)

    def get_char_age(self, cp):
        return self._icu.charAge(cp)

    def get_script(self, cp, alpha4=False):
        if alpha4:
            prop_type = U_SHORT_PROPERTY_NAME
        else:
            prop_type = U_LONG_PROPERTY_NAME
        return self._icu.get_script(cp, prop_type)

    def is_combining_mark(self, cp):
        return self._icu.is_combining_mark(cp)

    def is_digit(self, cp):
        return self._icu.is_digit(cp)

    def is_script_rtl(self, script):
        return self._icu.is_script_rtl(script)

    def get_script_extensions(self, cp, alpha4=False):
        if alpha4:
            prop_type = U_SHORT_PROPERTY_NAME
        else:
            prop_type = U_LONG_PROPERTY_NAME
        return self._icu.get_script_extensions(cp, prop_type)

    def get_prop_value(self, cp, prop_name, prop_type=U_LONG_PROPERTY_NAME):
        return self._icu.get_prop_value(cp, prop_name, prop_type)

    def compile_regex(self, regex):
        # return an ICU regex object
        return self._icu.re.compile(regex)

    def get_set(self, iterable=None, pattern=None, freeze=False):
        # return an ICU set object
        return self._icu.set(iterable, pattern, freeze)

    def idna_encode(self, input, options=None):
        return self._icu.idna_encode(input, self._icu.open_uts46(options) if options is not None else None)

    def idna_decode(self, input, options=None):
        return self._icu.idna_decode(input, self._icu.open_uts46(options) if options is not None else None)

    def idna_encode_label(self, input, options=None):
        return self._icu.idna_encode_label(input, self._icu.open_uts46(options) if options is not None else None)

    def idna_decode_label(self, input, options=None):
        return self._icu.idna_decode_label(input, self._icu.open_uts46(options) if options is not None else None)



# -*- coding: utf-8 -*-
"""
idnatables.py - Provide access to the data from IANA's IDNA 2008 tables.
"""
from __future__ import unicode_literals

import logging

from .idna_tables_1000 import idna_tables_1000
from .idna_tables_900 import idna_tables_900
from .idna_tables_800 import idna_tables_800
from .idna_tables_700 import idna_tables_700
from .idna_tables_630 import idna_tables_630
from .idna_tables_620 import idna_tables_620
from .idna_tables_610 import idna_tables_610
from .idna_tables_600 import idna_tables_600
from .idna_tables_520 import idna_tables_520

logger = logging.getLogger(__name__)


IDNA_UNICODE_MAPPING = {
    '10.0.0': idna_tables_1000,
    '9.0.0': idna_tables_900,
    '8.0.0': idna_tables_800,
    '7.0.0': idna_tables_700,
    '6.3.0': idna_tables_630,
    '6.2.0': idna_tables_620,
    '6.1.0': idna_tables_610,
    '6.0.0': idna_tables_600,
    '5.2.0': idna_tables_520,
}

IDNA_PROPERTIES = [
    'PVALID',
    'CONTEXTO',
    'CONTEXTJ',
    'DISALLOWED',
    'UNASSIGNED'
]


def get_idna_property(cp, unicode_version='6.3.0'):
    """
    Get the IDNA 2008 properties for a given codepoint.

    :param cp: The codepoint to look for in the IDNA tables.
    :param unicode_version: Unicode version to use.
    :returns: One of the IDNA_PROPERTIES values.
    :raises ValueError: If unicode_version is not handled by any IDNA table.
    """
    if unicode_version not in IDNA_UNICODE_MAPPING.keys():
        logger.error("Invalid Unicode version '%s'", unicode_version)
        raise ValueError("Invalid Unicode version '{0}'".format(unicode_version))

    mapper = IDNA_UNICODE_MAPPING[unicode_version]

    for prop in IDNA_PROPERTIES:
        if prop == 'UNASSIGNED':
            continue
        if cp in mapper[prop]:
            return prop
    return 'UNASSIGNED'



#!/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
from io import open

setup(
    name="munidata",
    version='2.1.0',
    description="Provides an API for accessing multiple instances of Unicode data",
    license="TBD",
    author='Viagenie and Wil Tan',
    author_email='support@viagenie.ca',
    install_requires=["picu"],
    packages=find_packages(),
    long_description=open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    scripts=['tools/parse_idna_tables.py'],
    classifiers=[
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries'
    ]
)
